pub struct Application<T> { /* private fields */ }Expand description
The outcome of successful compilation. Contains one or more CompiledFheProgram.
Implementations
Sourceimpl<T> Application<T>where
T: Fhe,
impl<T> Application<T>where
T: Fhe,
Sourcepub fn params(&self) -> &Params
pub fn params(&self) -> &Params
Returns the Params suitable for running each contained CompiledFheProgram.
These parameters were chosen during compilation.
Remarks
If no fhe_program was specified, this function panics.
Sourcepub fn get_program<N>(&self, name: N) -> Option<&CompiledFheProgram>
👎Deprecated
pub fn get_program<N>(&self, name: N) -> Option<&CompiledFheProgram>
Gets the CompiledFheProgram with the given name or None if not present.
Sourcepub fn get_fhe_program<N>(&self, name: N) -> Option<&CompiledFheProgram>
pub fn get_fhe_program<N>(&self, name: N) -> Option<&CompiledFheProgram>
Gets the CompiledFheProgram with the given name or None if not present.
Sourcepub fn get_programs(
&self,
) -> impl Iterator<Item = (&String, &CompiledFheProgram)>
👎Deprecated
pub fn get_programs( &self, ) -> impl Iterator<Item = (&String, &CompiledFheProgram)>
Returns an iterator over all the compiled programs.
Deprecated
Please use get_fhe_programs instead.
Sourcepub fn get_fhe_programs(
&self,
) -> impl Iterator<Item = (&String, &CompiledFheProgram)>
pub fn get_fhe_programs( &self, ) -> impl Iterator<Item = (&String, &CompiledFheProgram)>
Returns an iterator over all the compiled programs.
Sourceimpl<T> Application<T>where
T: Zkp,
impl<T> Application<T>where
T: Zkp,
Sourcepub fn get_zkp_program<N>(&self, name: N) -> Option<&CompiledZkpProgram>
pub fn get_zkp_program<N>(&self, name: N) -> Option<&CompiledZkpProgram>
Returns the CompiledZkpProgram with the given name
or None if not present.
Sourcepub fn get_zkp_programs(
&self,
) -> impl Iterator<Item = (&String, &CompiledZkpProgram)>
pub fn get_zkp_programs( &self, ) -> impl Iterator<Item = (&String, &CompiledZkpProgram)>
Returns an iterator over all CompiledZkpPrograms.