pub trait FheProgramInputTrait: TryIntoPlaintext + TypeNameInstance { }Expand description
A trait that denotes this type can be used as an argument to an FHE program.
pub trait FheProgramInputTrait: TryIntoPlaintext + TypeNameInstance { }A trait that denotes this type can be used as an argument to an FHE program.